package com.scbfkj.uni.api;

import com.scbfkj.uni.library.RequestUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.service.SecurityService;
import com.scbfkj.uni.system.Config;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.*;

@Component
@Aspect
public class LogAop {

    @Resource
    private SecurityService securityService;

    private static List<Map<String, Object>> apiInfos;


    @Around(value = "within( com.scbfkj.uni.api.*)")
    public Object invokeAround(ProceedingJoinPoint joinPoint) {
//        当前时间戳
        long beforeTime = System.currentTimeMillis();

        try {

//        请求
            ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
//
            HttpServletRequest request = requestAttributes.getRequest();
            String uri = request.getRequestURI();
//        请求参数
            Object[] args = joinPoint.getArgs();
            if (Config.enable) {
                if (Objects.isNull(apiInfos) || apiInfos.isEmpty()) {
                    apiInfos = DataBase.query(Config.securityConnectionStr, "select * from apiinfo", new ArrayList<>() {{
                        add(new Object[]{});
                    }});
                }

                Optional<Map<String, Object>> requestpath = apiInfos.stream().filter(it -> {
                    Object o = it.get("requestpath");
                    return Objects.equals(o, uri);
                }).findAny();
                if (requestpath.isPresent()) {
                    Map<String, Object> stringObjectMap = requestpath.get();
                    Object o = stringObjectMap.get("securitykey");
                    if (Objects.nonNull(o)) {
                        if (Objects.equals(o, "token")) {
                            String appToken = RequestUtil.getAppToken();
                            if(Objects.isNull(appToken)){
                                return ResponseEntity.ok(UniReturnUtil.fail("没有找到token"));
                            }
                            Map<String, Object> result = securityService.verifyToken(appToken);
                            if (!result.get("code").equals("0")) {
                                return result;
                            }
                        }
                        if (Objects.equals("usertoken", o)) {
                            String userToken = RequestUtil.getUserToken();
                            if(Objects.isNull(userToken)){
                                return ResponseEntity.ok(UniReturnUtil.fail("没有找到 usertoken"));
                            }
                            Map<String, Object> result = securityService.checkUserToken(userToken);
                            if (!result.get("code").equals("0")) {
                                return result;
                            }
                        }
                    }
                }
            }
            return joinPoint.proceed(args);
        } catch (Throwable e) {
            e.printStackTrace();
//            错误异常消息
            return ResponseEntity.ok(UniReturnUtil.fail(e.getMessage()));
        } finally {
            //        执行后时间戳
            long afterTime = System.currentTimeMillis();
//        执行时间
            long execTime = afterTime - beforeTime;
            System.out.println("执行时长：" + execTime);
//            todo 写日志
        }
    }
}
